package espresso;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

class ClassDef extends Def {
   private static final Name periodS = Name.fromString(".");
   private static final Name emptyS = Name.fromString("");
   ASTS closures;
   Scope funscope;
   FunObj clinit;
   Env clinitEnv;
   Code clinitcode;
   Code initcode;
   Block body;
   AST[] implementing;
   AST extending;

   void print(int var1) {
      if ((super.mods & 512) != 0) {
         System.out.print(Obj.modNames(super.mods ^ 512));
         System.out.print(String.valueOf("interface ").concat(String.valueOf(super.name)));
         if (this.implementing.length > 0) {
            System.out.print(" extends ");
            AST.printSeq(this.implementing);
         }
      } else {
         System.out.print(Obj.modNames(super.mods));
         System.out.print(String.valueOf("class ").concat(String.valueOf(super.name)));
         if (this.extending != null) {
            System.out.print(" extends ");
            this.extending.print();
         }

         if (this.implementing.length > 0) {
            System.out.print(" implements ");
            AST.printSeq(this.implementing);
         }
      }

      System.out.print(" ");
      this.body.print();
   }

   Name fullName() {
      return ((ClassObj)super.obj).fullname;
   }

   void output() {
      if (Report.nerrors == 0) {
         ClassObj var1 = (ClassObj)super.obj;

         try {
            File var2 = ClassFile.openOutput(var1);
            FileOutputStream var3 = new FileOutputStream(var2);
            ClassFile.writeClass(var3, var1);
            var3.close();
            if (Switches.verbose) {
               System.out.println(String.valueOf(String.valueOf("[wrote ").concat(String.valueOf(var2.getPath()))).concat(String.valueOf("]")));
            }
         } catch (IOException var5) {
            Report.error(super.pos, String.valueOf(String.valueOf(String.valueOf("error writing ").concat(String.valueOf(var1.name))).concat(String.valueOf(".class: "))).concat(String.valueOf(var5)));
         }
      }

   }

   Item gen() {
      this.genTry(false);
      return Item.voidItem;
   }

   private void genTry(boolean var1) {
      ClassObj var2 = (ClassObj)super.obj;
      var2.pool = Gen.newPool();
      this.initcode = Gen.newCode(var2, false, var1);
      this.clinitcode = Gen.newCode(var2, true, var1);

      for(int var3 = 0; var3 < this.body.stats.length; ++var3) {
         if (this.body.stats[var3].tag != 65) {
            this.body.stats[var3].genDrop();
         }
      }

      for(int var4 = 0; var4 < this.body.stats.length; ++var4) {
         if (this.body.stats[var4].tag == 65) {
            this.body.stats[var4].genDrop();
         }
      }

      if (this.clinitcode.cp != 0) {
         Gen.useCode(this.clinitcode);
         Gen.entryPoint(0);
         Gen.emitop(177);
         this.clinit.code = this.clinitcode;
         super.localEnv.scope.enter(this.clinit);
      }

      if (!var1 && (this.initcode.fatCode || this.clinitcode.fatCode)) {
         boolean var5 = Report.silent;
         Report.silent = true;
         this.genTry(true);
         Report.silent = var5;
      }

   }

   AST simplify() {
      this.body.markCaptured(new Bits());
      this.body.markCaptured(new Bits());
      if (this.extending != null) {
         this.extending = this.extending.simplify();
      }

      for(int var1 = 0; var1 < this.implementing.length; ++var1) {
         this.implementing[var1] = this.implementing[var1].simplify();
      }

      int var2 = this.body.pos;
      this.closures = new ASTS();
      this.body = (Block)this.body.simplify();
      ClassObj var3 = (ClassObj)super.obj;
      if (this.closures.length != 0) {
         super.localEnv.toplevel.newClasses.append(this.makeClosureClass());
         ASTS var4 = new ASTS(this.body.stats);

         for(int var5 = 0; var5 < this.closures.length; ++var5) {
            ((FunDef)this.closures.elems[var5]).obj = null;
            var4.append(this.closures.elems[var5]);
         }

         this.body = new Block(var2, var4);
      }

      return this;
   }

   private AST makeClosureClass() {
      int var1 = this.body.pos;
      AST[] var2 = new AST[]{new VarDef(var1, Simplify.fvarsS, 2, Simplify.objectArray(var1), (AST)null), new VarDef(var1, Simplify.receiverS, 2, new Ident(var1, super.name), (AST)null), new VarDef(var1, Simplify.tagS, 2, Simplify.toAST(var1, Typ.intTyp), (AST)null), this.makeClosureConstructor(), this.makeClosureApply()};
      return new ClassDef(var1, super.name.append(Simplify.closureS), 0, Simplify.toAST(var1, Simplify.ClosureS), new ASTS(), new Block(var1, new ASTS(var2)));
   }

   private AST makeClosureConstructor() {
      int var1 = this.body.pos;
      return new FunDef(var1, 65, Predef.initS, 0, (AST)null, new ASTS(new VarDef(var1, Simplify.fvarsS, 0, Simplify.objectArray(var1), (AST)null), new VarDef(var1, Simplify.receiverS, 0, new Ident(var1, super.name), (AST)null), new VarDef(var1, Simplify.tagS, 0, Simplify.toAST(var1, Typ.intTyp), (AST)null)), new ASTS(), new Block(var1, new ASTS(new Exec(var1, new Assignop(var1, 49, Simplify.selectThis(var1, Simplify.fvarsS), new Ident(var1, Simplify.fvarsS))), new Exec(var1, new Assignop(var1, 49, Simplify.selectThis(var1, Simplify.receiverS), new Ident(var1, Simplify.receiverS))), new Exec(var1, new Assignop(var1, 49, Simplify.selectThis(var1, Simplify.tagS), new Ident(var1, Simplify.tagS))))));
   }

   private AST makeClosureApply() {
      int var1 = this.body.pos;
      ASTS var2 = new ASTS();
      ClassObj var3 = (ClassObj)super.obj;
      int var4 = Simplify.totalClosures(var3.superclass);

      for(int var5 = 0; var5 < this.closures.length; ++var5) {
         var2.append(this.makeClosureCall((FunDef)this.closures.elems[var5], var5 + var4));
      }

      AST[] var6 = new AST[]{new VarDef(var1, Simplify.allargsS, 0, Simplify.objectArray(var1), new Apply(var1, new Select(var1, Simplify.toAST(var1, Simplify.ClosureS), Simplify.combineS), new ASTS(new Ident(var1, Simplify.fvarsS), new Ident(var1, Simplify.argsS)))), new Switch(var1, new Ident(var1, Simplify.tagS), var2), new Throw(var1, new New(var1, new Apply(var1, Simplify.toAST(var1, Simplify.MessageNotUnderstoodS), new ASTS(new Ident(var1, Simplify.tagS)))))};
      return new FunDef(var1, 65, Simplify.applyS, 1, Simplify.objectTyp(var1), new ASTS(new VarDef(var1, Simplify.argsS, 0, Simplify.objectArray(var1), (AST)null)), new ASTS(), new Block(var1, new ASTS(var6)));
   }

   private AST makeClosureCall(FunDef var1, int var2) {
      int var3 = this.body.pos;
      FunTyp var4 = (FunTyp)var1.typ;
      ASTS var5 = new ASTS();
      Hashtable var6 = ((FunObj)var1.obj).freevars;
      Enumeration var7 = var6.elements();
      int var8 = var6.size();

      for(int var9 = 0; var9 < var1.params.length; ++var9) {
         Object var10;
         if (var9 < var8) {
            VarObj var11 = (VarObj)var7.nextElement();
            var10 = var11.typ;
            if (var11.passByRef) {
               var10 = new ArrayTyp((Typ)var10);
            }
         } else {
            var10 = var4.argtyps[var9 - var8];
         }

         var5.append(Simplify.unbox(new Subscript(var3, new Ident(var3, Simplify.allargsS), new Literal(var3, 90, new Integer(var9))), (Typ)var10));
      }

      Name var12 = (var1.mods & 8) != 0 ? super.name : Simplify.receiverS;
      Apply var13 = new Apply(var3, Simplify.toAST(var3, var12.append(periodS).append(var1.name)), var5);
      ASTS var14 = var4.restyp.tag == 12 ? new ASTS(new Exec(var3, var13), new Return(var3, new Ident(var3, Simplify.nullS))) : new ASTS(new Return(var3, Simplify.box(var13)));
      return new Case(var3, new Literal(var3, 90, new Integer(var2)), var14);
   }

   Typ attr(Env var1, int var2, Typ var3) {
      ClassObj var4 = (ClassObj)super.obj;
      if (!this.fatallyFlawed(var4)) {
         Attr.fixupScope(super.pos, var4);
         this.funscope.baseOn(var4.scope);
         this.clinit = new FunObj(8, Predef.clinitS, new FunTyp(new Typ[0], Typ.voidTyp, (TypSet)null));
         this.clinitEnv = new Env(super.localEnv, this, new Scope(super.localEnv.scope, this.clinit));
         this.clinitEnv.isStatic = true;
         if ((var4.modifiers & 1536) == 0) {
            var4.modifiers ^= 1024;
            Obj var5 = firstUndef(var4, var4);
            if (var5 != null) {
               Report.error(super.pos, String.valueOf(String.valueOf(String.valueOf(String.valueOf(var4).concat(String.valueOf(" should be declared abstract; "))).concat(String.valueOf("it does not define "))).concat(String.valueOf(var5))).concat(String.valueOf(Attr.location(var5))));
            }

            var4.modifiers ^= 1024;
         }

         this.body.typ = Typ.voidTyp;

         for(int var6 = 0; var6 < this.body.stats.length; ++var6) {
            this.body.stats[var6].attr(super.localEnv, 12, Typ.voidTyp);
         }
      }

      super.typ = var3;
      return super.typ;
   }

   void enterFields(Env var1) {
      ClassObj var2 = (ClassObj)super.obj;
      var2.scope = new Scope((Scope)null, var2);
      if (this.extending == null) {
         if (var2.isRoot()) {
            var2.superclass = null;
         } else {
            var2.superclass = Predef.objectClass;
         }
      } else {
         var2.superclass = this.classRef(var1, this.extending, true);
      }

      var2.interfaces = new ClassObj[this.implementing.length];

      for(int var3 = 0; var3 < var2.interfaces.length; ++var3) {
         var2.interfaces[var3] = this.classRef(var1, this.implementing[var3], false);
      }

      this.funscope = new Scope(var2.scope, (Obj)null);
      super.localEnv = new Env(var1, this, var2.scope);
      super.localEnv.enclClass = this;
      this.ensureConstructor();
      if (!this.fatallyFlawed(var2)) {
         for(int var4 = 0; var4 < this.body.stats.length; ++var4) {
            this.body.stats[var4].enter(super.localEnv);
         }
      }

   }

   private boolean fatallyFlawed(ClassObj var1) {
      if (var1.superclass != null && var1.superclass.typ.tag == 0) {
         return true;
      } else {
         for(int var2 = 0; var2 < var1.interfaces.length; ++var2) {
            if (var1.interfaces[var2].typ.tag == 0) {
               return true;
            }
         }

         return false;
      }
   }

   private void ensureConstructor() {
      if ((super.mods & 512) == 0 && !this.body.hasConstructors()) {
         this.body.pushStat(new FunDef(super.pos, 65, Predef.initS, 1, (AST)null, new ASTS(), new ASTS(), new Block(super.pos, new ASTS())));
      }

   }

   private ClassObj classRef(Env var1, AST var2, boolean var3) {
      ClassObj var4 = Attr.checkObjectTyp(var2.pos, var2.attr(var1, 2, Typ.anyTyp)).obj;
      if (var4.typ.tag == 11) {
         if (var3 && (var4.modifiers & 512) != 0) {
            Report.error(var2.pos, "no interface allowed here");
         } else if (!var3 && (var4.modifiers & 512) == 0) {
            Report.error(var2.pos, "not an interface");
         } else if ((var4.modifiers & 16) != 0) {
            Report.error(var2.pos, String.valueOf("can't inherit from final ").concat(String.valueOf(var4)));
         }
      }

      return var4;
   }

   void enter(Env var1) {
      ClassObj var2 = new ClassObj();
      var2.name = super.name;
      var2.modifiers = Attr.checkMods(super.pos, super.mods, 1553);
      var2.typ = new Typ(11, var2);
      var2.fullname = var1.packageName.append(super.name);
      var2.packagename = var1.packageName;
      var2.sourcefile = Name.fromString(var1.toplevel.sourcefile);
      super.obj = var2;
      if (var1.scope.lookup(super.name).scope == var1.scope) {
         Report.error(super.pos, String.valueOf("duplicate definition of class ").concat(String.valueOf(var2.fullname)));
      } else {
         var1.scope.enter(var2);
         ClassFile.loaded.put(var2.fullname, var2);
      }

   }

   private static Obj firstUndef(ClassObj var0, ClassObj var1) {
      Obj var2 = null;
      if ((var1.modifiers & 1536) != 0) {
         var2 = firstUndef(var0, var1.scope);
         if (var2 == null && var1.superclass != null) {
            var2 = firstUndef(var0, var1.superclass);
         }

         if (var2 == null && var1.interfaces.length > 0) {
            var2 = firstUndef(var0, var1.scope.next);
         }
      }

      return var2;
   }

   private static Obj firstUndef(ClassObj var0, Scope var1) {
      Object var2 = null;

      for(ScopeEntry var3 = var1.elements(); var3 != null; var3 = var3.sibling) {
         if (var3.obj instanceof FunObj && (var3.obj.modifiers & 1024) != 0) {
            ScopeEntry var4;
            for(var4 = var0.scope.lookup(var3.obj.name); var4.scope != null && (!(var4.obj instanceof FunObj) || (var4.obj.modifiers & 1024) != 0 || !((FunObj)var4.obj).overrides(var3.obj)); var4 = var4.next()) {
            }

            if (var4.scope == null) {
               return var3.obj;
            }
         }
      }

      return null;
   }

   ClassDef(int var1, Name var2, int var3, AST var4, ASTS var5, Block var6) {
      super(var1, 16, var2, var3);
      this.extending = var4;
      this.implementing = var5.shrink();
      this.body = var6;
   }
}
